@@ -232,7 +232,7 @@ NS_IMETHODIMP EwsFolder::CreateSubfolder(const nsACString& aFolderName,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<IEwsClient> client;
-  rv = CreateProtocolClient(getter_AddRefs(client));
+  rv = GetProtocolClient(getter_AddRefs(client));
   NS_ENSURE_SUCCESS(rv, rv);
 
   const auto folderName = nsCString(aFolderName);
@@ -360,7 +360,7 @@ NS_IMETHODIMP EwsFolder::RenameSubFolders(nsIMsgWindow* msgWindow,
 NS_IMETHODIMP EwsFolder::MarkMessagesRead(
     const nsTArray<RefPtr<nsIMsgDBHdr>>& messages, bool markRead) {
   nsCOMPtr<IEwsClient> client;
-  nsresult rv = CreateProtocolClient(getter_AddRefs(client));
+  nsresult rv = GetProtocolClient(getter_AddRefs(client));
   NS_ENSURE_SUCCESS(rv, rv);
 
   CopyableTArray<nsCString> requestedIds(messages.Length());
@@ -419,7 +419,7 @@ NS_IMETHODIMP EwsFolder::MarkMessagesRead(
 
 NS_IMETHODIMP EwsFolder::MarkAllMessagesRead(nsIMsgWindow* aMsgWindow) {
   nsCOMPtr<IEwsClient> client;
-  MOZ_TRY(CreateProtocolClient(getter_AddRefs(client)));
+  MOZ_TRY(GetProtocolClient(getter_AddRefs(client)));
 
   nsCString folderId;
   MOZ_TRY(GetEwsId(folderId));
@@ -518,7 +518,7 @@ NS_IMETHODIMP EwsFolder::Rename(const nsACString& aNewName,
   }
 
   nsCOMPtr<IEwsClient> client;
-  rv = CreateProtocolClient(getter_AddRefs(client));
+  rv = GetProtocolClient(getter_AddRefs(client));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsAutoCString folderId;
@@ -559,7 +559,7 @@ NS_IMETHODIMP EwsFolder::CopyFileMessage(
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<IEwsClient> client;
-  rv = CreateProtocolClient(getter_AddRefs(client));
+  rv = GetProtocolClient(getter_AddRefs(client));
   NS_ENSURE_SUCCESS(rv, rv);
 
   RefPtr<MessageCopyHandler> handler = new MessageCopyHandler(
@@ -618,7 +618,7 @@ NS_IMETHODIMP EwsFolder::CopyMessages(
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<IEwsClient> client;
-    MOZ_TRY(CreateProtocolClient(getter_AddRefs(client)));
+    MOZ_TRY(GetProtocolClient(getter_AddRefs(client)));
 
     RefPtr<MessageCopyHandler> handler =
         new MessageCopyHandler(aSrcFolder, this, aSrcHdrs, aIsMove, aMsgWindow,
@@ -754,7 +754,7 @@ NS_IMETHODIMP EwsFolder::CopyItemsOnSameServer(
           });
 
   nsCOMPtr<IEwsClient> client;
-  MOZ_TRY(CreateProtocolClient(getter_AddRefs(client)));
+  MOZ_TRY(GetProtocolClient(getter_AddRefs(client)));
 
   if (aIsMove) {
     rv = client->MoveItems(listener, destinationFolderId, ewsIds);
@@ -801,7 +801,7 @@ NS_IMETHODIMP EwsFolder::CopyFolder(nsIMsgFolder* aSrcFolder,
   auto notifyFailureOnExit = GuardCopyServiceExit(aSrcFolder, this, rv);
 
   nsCOMPtr<IEwsClient> client;
-  MOZ_TRY(CreateProtocolClient(getter_AddRefs(client)));
+  MOZ_TRY(GetProtocolClient(getter_AddRefs(client)));
 
   bool isSameServer;
   rv = FoldersOnSameServer(aSrcFolder, this, &isSameServer);
@@ -907,7 +907,7 @@ NS_IMETHODIMP EwsFolder::CopyFolderOnSameServer(
       });
 
   nsCOMPtr<IEwsClient> client;
-  MOZ_TRY(CreateProtocolClient(getter_AddRefs(client)));
+  MOZ_TRY(GetProtocolClient(getter_AddRefs(client)));
 
   if (aIsMoveFolder) {
     rv = client->MoveFolders(listener, destinationEwsId, {sourceEwsId});
@@ -1027,7 +1027,7 @@ NS_IMETHODIMP EwsFolder::DeleteMessages(
         });
 
     nsCOMPtr<IEwsClient> client;
-    rv = self->CreateProtocolClient(getter_AddRefs(client));
+    rv = self->GetProtocolClient(getter_AddRefs(client));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsTArray<nsCString> ewsIds;
@@ -1072,7 +1072,7 @@ NS_IMETHODIMP EwsFolder::DeleteSelf(nsIMsgWindow* aWindow) {
         });
 
     nsCOMPtr<IEwsClient> client;
-    nsresult rv = self->CreateProtocolClient(getter_AddRefs(client));
+    nsresult rv = self->GetProtocolClient(getter_AddRefs(client));
     NS_ENSURE_SUCCESS(rv, rv);
     return client->DeleteFolder(listener, {folderId});
   };
@@ -1177,7 +1177,7 @@ NS_IMETHODIMP EwsFolder::EmptyTrash(nsIUrlListener* aListener) {
       });
 
   nsCOMPtr<IEwsClient> client;
-  rv = CreateProtocolClient(getter_AddRefs(client));
+  rv = GetProtocolClient(getter_AddRefs(client));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = client->EmptyFolder(listener, {trashEwsId}, subFolderIds, messageIds);
@@ -1245,14 +1245,14 @@ nsresult EwsFolder::GetEwsId(nsACString& ewsId) {
   return NS_OK;
 }
 
-nsresult EwsFolder::CreateProtocolClient(IEwsClient** ewsClient) {
+nsresult EwsFolder::GetProtocolClient(IEwsClient** ewsClient) {
   nsCOMPtr<nsIMsgIncomingServer> server;
   nsresult rv = GetServer(getter_AddRefs(server));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<IEwsIncomingServer> ewsServer(do_QueryInterface(server));
 
-  return ewsServer->CreateProtocolClient(ewsClient);
+  return ewsServer->GetProtocolClient(ewsClient);
 }
 
 nsresult EwsFolder::GetTrashFolder(nsIMsgFolder** result) {
@@ -1401,6 +1401,7 @@ nsresult EwsFolder::SyncMessages(nsIMsgWindow* window,
 
     if (!(flags & nsMsgMessageFlags::Offline)) {
       // Bail early if there's nothing to remove.
+      existingHdr.forget(hdr);
       return NS_OK;
     }
 
@@ -1559,7 +1560,7 @@ nsresult EwsFolder::SyncMessages(nsIMsgWindow* window,
   MOZ_TRY(GetEwsId(ewsId));
 
   nsCOMPtr<IEwsClient> client;
-  MOZ_TRY(CreateProtocolClient(getter_AddRefs(client)));
+  MOZ_TRY(GetProtocolClient(getter_AddRefs(client)));
 
   return client->SyncMessagesForFolder(listener, ewsId, syncStateToken);
 }
@@ -1994,7 +1995,7 @@ NS_IMETHODIMP EwsFolder::HandleViewCommand(
             }};
 
     nsCOMPtr<IEwsClient> client;
-    rv = CreateProtocolClient(getter_AddRefs(client));
+    rv = GetProtocolClient(getter_AddRefs(client));
     NS_ENSURE_SUCCESS(rv, rv);
 
     rv = client->MarkItemsAsJunk(operationListener, ewsIds, isJunk,
