@@ -42,10 +42,10 @@ using mozilla::dom::Element;
 using mozilla::dom::WindowContext;
 
 // Reference to the pointer locked element.
-MOZ_CONSTINIT static nsWeakPtr sLockedElement;
+constinit static nsWeakPtr sLockedElement;
 
 // Reference to the document which requested pointer lock.
-MOZ_CONSTINIT static nsWeakPtr sLockedDoc;
+constinit static nsWeakPtr sLockedDoc;
 
 // Reference to the BrowserParent requested pointer lock.
 static BrowserParent* sLockedRemoteTarget = nullptr;
