@@ -24,7 +24,6 @@
 #include "XiphExtradata.h"
 #include "gfx2DGlue.h"
 #include "gfxUtils.h"
-#include "mozilla/EndianUtils.h"
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/SharedThreadPool.h"
@@ -439,8 +438,8 @@ nsresult WebMDemuxer::ReadMetadata() {
           FloorDefaultDurationToTimecodeScale(context, track);
       nsresult rv = SetVideoCodecInfo(context, track);
       if (NS_FAILED(rv)) {
-        WEBM_DEBUG("Set video codec info error");
-        return rv;
+        WEBM_DEBUG("Set video codec info error, ignoring track");
+        continue;
       }
       mInfo.mVideo.mColorPrimaries = gfxUtils::CicpToColorPrimaries(
           static_cast<gfx::CICP::ColourPrimaries>(params.primaries),
@@ -531,6 +530,13 @@ nsresult WebMDemuxer::ReadMetadata() {
                    params.rate, params.channels);
         return NS_ERROR_DOM_MEDIA_METADATA_ERR;
       }
+      params.rate = rate;
+
+      nsresult rv = SetAudioCodecInfo(context, track, params);
+      if (NS_FAILED(rv)) {
+        WEBM_DEBUG("Set audio codec info error, ignoring track");
+        continue;
+      }
 
       mAudioTrack = track;
       mHasAudio = true;
@@ -540,12 +546,6 @@ nsresult WebMDemuxer::ReadMetadata() {
       mInfo.mAudio.mRate = rate;
       mInfo.mAudio.mChannels = params.channels;
 
-      nsresult rv = SetAudioCodecInfo(context, track, params);
-      if (NS_FAILED(rv)) {
-        WEBM_DEBUG("Set audio codec info error");
-        return rv;
-      }
-
       uint64_t duration = 0;
       r = nestegg_duration(context, &duration);
       if (!r) {
@@ -561,6 +561,11 @@ nsresult WebMDemuxer::ReadMetadata() {
       }
     }
   }
+
+  if (!mHasVideo && !mHasAudio) {
+    WEBM_DEBUG("No supported track!");
+    return NS_ERROR_DOM_MEDIA_METADATA_ERR;
+  }
   WEBM_DEBUG("Read metadata OK");
   return NS_OK;
 }
@@ -825,7 +830,6 @@ nsresult WebMDemuxer::GetNextPacket(TrackInfo::TrackType aType,
             packetEncryption == NESTEGG_PACKET_HAS_SIGNAL_BYTE_UNENCRYPTED ||
                 packetEncryption == NESTEGG_PACKET_HAS_SIGNAL_BYTE_FALSE,
             "Unencrypted packet expected");
-
         sample->mKeyframe = CheckKeyFrameByExamineByteStream(sample);
       }
     }
@@ -921,6 +925,12 @@ nsresult WebMDemuxer::GetNextPacket(TrackInfo::TrackType aType,
 
           for (uint8_t i = 0; i < numPartitions; i++) {
             uint32_t partition = partitions[i];
+            if (partition > length || partition < lastOffset) {
+              WEBM_DEBUG(
+                  "Invalid partition offset: %u (length: %zu, lastOffset: %u)",
+                  partition, length, lastOffset);
+              return NS_ERROR_DOM_MEDIA_DEMUXER_ERR;
+            }
             uint32_t currentLength = partition - lastOffset;
 
             if (encrypted) {
@@ -931,8 +941,6 @@ nsresult WebMDemuxer::GetNextPacket(TrackInfo::TrackType aType,
 
             encrypted = !encrypted;
             lastOffset = partition;
-
-            MOZ_ASSERT(lastOffset <= length);
           }
 
           // Add the data between the last offset and the end of the data.
