@@ -10,10 +10,10 @@
 
 #include "Navigator.h"
 #include "NotificationUtils.h"
-#include "mozilla/Encoding.h"
 #include "mozilla/HoldDropJSObjects.h"
 #include "mozilla/OwningNonNull.h"
 #include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/UseCounter.h"
 #include "mozilla/dom/BindingUtils.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/Promise-inl.h"
@@ -134,6 +134,9 @@ NotificationPermissionRequest::Run() {
                  PermissionCheckPurpose::PermissionRequest,
                  mWindow->GetExtantDoc())) {
     mPermission = NotificationPermission::Denied;
+  } else if (!StaticPrefs::dom_webnotifications_allowcrossoriginiframe() &&
+             !mPrincipal->Subsumes(mTopLevelPrincipal)) {
+    mPermission = NotificationPermission::Denied;
   }
 
   // We can't call ShowPrompt() directly here since our logic for determining
@@ -343,6 +346,16 @@ static Result<nsString, nsresult> SerializeDataAsBase64(
   return result;
 }
 
+#define SetUseCounterIf(wasUsed, memberName)                             \
+  if (wasUsed) {                                                         \
+    if (NS_IsMainThread()) {                                             \
+      SetUseCounter(aGlobal->GetGlobalJSObject(),                        \
+                    eUseCounter_NotificationOptions_##memberName);       \
+    } else {                                                             \
+      SetUseCounter(UseCounterWorker::NotificationOptions_##memberName); \
+    }                                                                    \
+  }
+
 /* static */
 // https://notifications.spec.whatwg.org/#create-a-notification
 already_AddRefed<Notification> Notification::ValidateAndCreate(
@@ -351,6 +364,15 @@ already_AddRefed<Notification> Notification::ValidateAndCreate(
     ErrorResult& aRv) {
   MOZ_ASSERT(aGlobal);
 
+  SetUseCounterIf(aOptions.mNavigate.WasPassed(), navigate);
+  SetUseCounterIf(aOptions.mImage.WasPassed(), image);
+  SetUseCounterIf(aOptions.mBadge.WasPassed(), badge);
+  SetUseCounterIf(aOptions.mVibrate.WasPassed(), vibrate);
+  SetUseCounterIf(aOptions.mTimestamp.WasPassed(), timestamp);
+  SetUseCounterIf(aOptions.mRenotify, renotify);
+  SetUseCounterIf(aOptions.mRequireInteraction, requireInteraction);
+  SetUseCounterIf(!aOptions.mActions.IsEmpty(), actions);
+
   // Step 4: Set notification’s data to
   // StructuredSerializeForStorage(options["data"]).
   JS::Rooted<JS::Value> data(aCx, aOptions.mData);
@@ -393,8 +415,7 @@ already_AddRefed<Notification> Notification::ValidateAndCreate(
   // Step 12: If options["icon"] exists, then parse it using baseURL, and if
   // that does not return failure, set notification’s icon URL to the return
   // value. (Otherwise icon URL is not set.)
-  nsAutoString iconUrl;
-  ResolveIconURL(aGlobal, aOptions.mIcon, iconUrl);
+  RefPtr<nsIURI> iconUrl = ResolveIconURL(aGlobal, aOptions.mIcon);
 
   // Step 19: Set notification’s actions to « ».
   nsTArray<IPCNotificationAction> actions;
@@ -423,7 +444,7 @@ already_AddRefed<Notification> Notification::ValidateAndCreate(
                       nsString(aTitle), aOptions.mDir, nsString(aOptions.mLang),
                       nsString(aOptions.mBody), nsString(aOptions.mTag),
                       iconUrl, aOptions.mRequireInteraction, silent, vibrate,
-                      nsString(dataResult.unwrap()), std::move(actions)));
+                      nsString(dataResult.unwrap()), actions));
 
   RefPtr<Notification> notification =
       new Notification(aGlobal, ipcNotification, aScope);
@@ -561,85 +582,26 @@ uint32_t Notification::MaxActions(const GlobalObject& aGlobal) {
   return kMaxActions;
 }
 
-nsresult Notification::ResolveIconURL(nsIGlobalObject* aGlobal,
-                                      const nsAString& aIconUrl,
-                                      nsString& aDecodedUrl) {
+already_AddRefed<nsIURI> Notification::ResolveIconURL(
+    nsIGlobalObject* aGlobal, const nsACString& aIconUrl) {
   nsresult rv = NS_OK;
 
   if (aIconUrl.IsEmpty()) {
-    return rv;
+    return nullptr;
   }
 
   nsCOMPtr<nsIURI> baseUri = aGlobal->GetBaseURI();
   if (!baseUri) {
-    return rv;
-  }
-
-  auto encoding = UTF_8_ENCODING;
-
-  // TODO(krosylight): Ultimately we can use UTF8String for icon definition of
-  // Web IDL when we remove this.
-  if (!StaticPrefs::dom_webnotifications_icon_encoding_utf8_enabled()) {
-    if (nsCOMPtr<nsPIDOMWindowInner> window = aGlobal->GetAsInnerWindow()) {
-      if (RefPtr<Document> doc = window->GetExtantDoc()) {
-        encoding = doc->GetDocumentCharacterSet();
-      } else {
-        NS_WARNING("No document found for main thread notification!");
-        return NS_ERROR_FAILURE;
-      }
-    }
+    return nullptr;
   }
 
   nsCOMPtr<nsIURI> srcUri;
-  rv = NS_NewURI(getter_AddRefs(srcUri), aIconUrl, encoding, baseUri);
-  if (NS_SUCCEEDED(rv)) {
-    nsAutoCString src;
-    srcUri->GetSpec(src);
-    // XXX(krosylight): We should be able to pass UTF8 as-is, or ideally the URI
-    // object itself.
-    CopyUTF8toUTF16(src, aDecodedUrl);
+  rv = NS_NewURI(getter_AddRefs(srcUri), aIconUrl, nullptr, baseUri);
+  if (NS_FAILED(rv)) {
+    return nullptr;
   }
 
-  // TODO(krosylight): We should be able to remove the following when removing
-  // the non-UTF8 branch above.
-  if (encoding == UTF_8_ENCODING) {
-    return rv;
-  }
-
-  // If it was not UTF8, let's try UTF8 and see whether the result differs. If
-  // no difference is found then we can just use UTF8 everywhere.
-  // See: https://github.com/whatwg/notifications/issues/209
-  glean::web_notification::IconUrlEncodingLabel label =
-      glean::web_notification::IconUrlEncodingLabel::eNeitherWay;
-
-  nsCOMPtr<nsIURI> srcUriUtf8;
-  nsresult rvUtf8 =
-      NS_NewURI(getter_AddRefs(srcUriUtf8), aIconUrl, UTF_8_ENCODING, baseUri);
-
-  if (NS_SUCCEEDED(rv)) {
-    if (NS_SUCCEEDED(rvUtf8)) {
-      bool equals = false;
-      if (NS_SUCCEEDED(srcUri->Equals(srcUriUtf8, &equals))) {
-        if (equals) {
-          // Okay to be parsed with UTF8
-          label = glean::web_notification::IconUrlEncodingLabel::eUtf8;
-        } else {
-          // Can be parsed either way but with difference, unclear which one is
-          // intended without fetching
-          label = glean::web_notification::IconUrlEncodingLabel::eEitherWay;
-        }
-      }
-    } else {
-      label = glean::web_notification::IconUrlEncodingLabel::eDocumentCharset;
-    }
-  } else if (NS_SUCCEEDED(rvUtf8)) {
-    // Can be only parsed with UTF8
-    label = glean::web_notification::IconUrlEncodingLabel::eUtf8;
-  }
-
-  glean::web_notification::icon_url_encoding.EnumGet(label).Add();
-
-  return rv;
+  return srcUri.forget();
 }
 
 JSObject* Notification::WrapObject(JSContext* aCx,
