@@ -292,7 +292,7 @@ static inline bool IsString(const TrustedTypeOrStringArg& aInput) {
     return aInput.IsUSVString();
   }
   if constexpr (std::is_same_v<TrustedTypeOrStringArg, const nsAString*>) {
-    Unused << aInput;
+    (void)aInput;
     return true;
   }
   MOZ_ASSERT_UNREACHABLE();
@@ -352,7 +352,7 @@ static inline bool IsTrustedType(const TrustedTypeOrStringArg& aInput) {
     return aInput.IsTrustedScriptURL();
   }
   if constexpr (std::is_same_v<TrustedTypeOrStringArg, const nsAString*>) {
-    Unused << aInput;
+    (void)aInput;
     return false;
   }
   MOZ_ASSERT_UNREACHABLE();
@@ -396,7 +396,7 @@ static inline const nsAString* GetAsTrustedType(
     MOZ_ASSERT(aInput.IsTrustedScriptURL());
     return &aInput.GetAsTrustedScriptURL().mData;
   }
-  Unused << aInput;
+  (void)aInput;
   MOZ_ASSERT_UNREACHABLE();
   return &EmptyString();
 };
@@ -407,29 +407,23 @@ static inline const nsAString* GetContent(
   return IsString(aInput) ? GetAsString(aInput) : GetAsTrustedType(aInput);
 }
 
-template <typename ExpectedType, typename TrustedTypeOrString,
-          typename NodeOrGlobalObject>
-MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
-    const TrustedTypeOrString& aInput, const nsAString& aSink,
-    const nsAString& aSinkGroup, NodeOrGlobalObject& aNodeOrGlobalObject,
-    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+template <typename NodeOrGlobalObject>
+inline bool PrepareExecutionOfTrustedTypesDefaultPolicy(
+    NodeOrGlobalObject& aNodeOrGlobalObject, nsIPrincipal* aPrincipalOrNull,
+    nsIGlobalObject*& aGlobalObject, nsPIDOMWindowInner*& aPIDOMWindowInner,
+    RequireTrustedTypesForDirectiveState* aRequireTrustedTypesForDirectiveState,
     ErrorResult& aError) {
-  MOZ_ASSERT(aSinkGroup == kTrustedTypesOnlySinkGroup);
   if (!StaticPrefs::dom_security_trusted_types_enabled()) {
     // A trusted type might've been created before the pref was set to `false`,
     // so we cannot assume aInput.IsString().
-    return GetContent(aInput);
-  }
-
-  if (IsTrustedType(aInput)) {
-    return GetAsTrustedType(aInput);
+    return false;
   }
 
   // Exempt web extension content scripts from trusted types policies defined by
   // the page in which they are running.
   if (auto* principal = BasePrincipal::Cast(aPrincipalOrNull)) {
     if (principal->ContentScriptAddonPolicyCore()) {
-      return GetAsString(aInput);
+      return false;
     }
   }
 
@@ -443,28 +437,31 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
   nsIGlobalObject* globalObject = nullptr;
   nsPIDOMWindowInner* piDOMWindowInner = nullptr;
   if constexpr (std::is_same_v<NodeOrGlobalObjectArg, nsINode>) {
+    if (aNodeOrGlobalObject.HasBeenInUAWidget()) {
+      return false;
+    }
     Document* ownerDoc = aNodeOrGlobalObject.OwnerDoc();
     const bool ownerDocLoadedAsData = ownerDoc->IsLoadedAsData();
     if (!ownerDoc->HasPolicyWithRequireTrustedTypesForDirective() &&
         !ownerDocLoadedAsData) {
-      return GetAsString(aInput);
+      return false;
     }
     globalObject = ownerDoc->GetScopeObject();
     if (!globalObject) {
       aError.ThrowTypeError("No global object");
-      return nullptr;
+      return false;
     }
     piDOMWindowInner = globalObject->GetAsInnerWindow();
     if (!piDOMWindowInner) {
       // Global object is not a Window. This can happen when DOM APIs are used
       // in some contexts where Trusted Types don't apply (e.g. bug 1942517),
       // so just return the input string.
-      return GetAsString(aInput);
+      return false;
     }
     if (ownerDocLoadedAsData && piDOMWindowInner->GetExtantDoc() &&
         !piDOMWindowInner->GetExtantDoc()
              ->HasPolicyWithRequireTrustedTypesForDirective()) {
-      return GetAsString(aInput);
+      return false;
     }
   } else if constexpr (std::is_same_v<NodeOrGlobalObjectArg, nsIGlobalObject>) {
     piDOMWindowInner = aNodeOrGlobalObject.GetAsInnerWindow();
@@ -472,7 +469,7 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
       const Document* extantDoc = piDOMWindowInner->GetExtantDoc();
       if (extantDoc &&
           !extantDoc->HasPolicyWithRequireTrustedTypesForDirective()) {
-        return GetAsString(aInput);
+        return false;
       }
     }
     globalObject = &aNodeOrGlobalObject;
@@ -486,13 +483,13 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
   // (https://w3c.github.io/trusted-types/dist/spec/#abstract-opdef-does-sink-type-require-trusted-types)
   // and "Should sink type mismatch violation be blocked by CSP?"
   // (https://w3c.github.io/trusted-types/dist/spec/#should-block-sink-type-mismatch).
-  RefPtr<nsIContentSecurityPolicy> csp;
   RequireTrustedTypesForDirectiveState requireTrustedTypesForDirectiveState =
       RequireTrustedTypesForDirectiveState::NONE;
   if (piDOMWindowInner) {
-    csp = PolicyContainer::GetCSP(piDOMWindowInner->GetPolicyContainer());
+    RefPtr<nsIContentSecurityPolicy> csp =
+        PolicyContainer::GetCSP(piDOMWindowInner->GetPolicyContainer());
     if (!csp) {
-      return GetAsString(aInput);
+      return false;
     }
     requireTrustedTypesForDirectiveState =
         csp->GetRequireTrustedTypesForDirectiveState();
@@ -508,13 +505,53 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
         cspInfo.requireTrustedTypesForDirectiveState();
     if (requireTrustedTypesForDirectiveState ==
         RequireTrustedTypesForDirectiveState::NONE) {
-      return GetAsString(aInput);
+      return false;
     }
   } else {
     // Global object is neither Window nor WorkerGlobalScope. This can happen
     // when DOM APIs are used in some contexts where Trusted Types don't apply
     // (e.g. bugs 1942517 and 1936219), so just return the input string.
-    return GetAsString(aInput);
+    return false;
+  }
+
+  aGlobalObject = globalObject;
+  aPIDOMWindowInner = piDOMWindowInner;
+  *aRequireTrustedTypesForDirectiveState = requireTrustedTypesForDirectiveState;
+  return true;
+}
+
+bool CanSkipTrustedTypesEnforcement(const nsINode& aNode) {
+  nsIGlobalObject* dummyGlobal = nullptr;
+  nsPIDOMWindowInner* dummyWindow = nullptr;
+  RequireTrustedTypesForDirectiveState dummyState;
+  // Specify a null principal, as ScriptElement::UpdateTrustWorthiness() has
+  // already checked whether this is running in an isolated web extension
+  // context.
+  return !PrepareExecutionOfTrustedTypesDefaultPolicy(
+      aNode, nullptr /* aPrincipalOrNull */, dummyGlobal, dummyWindow,
+      &dummyState, IgnoreErrors());
+}
+
+template <typename ExpectedType, typename TrustedTypeOrString,
+          typename NodeOrGlobalObject>
+MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
+    const TrustedTypeOrString& aInput, const nsAString& aSink,
+    const nsAString& aSinkGroup, NodeOrGlobalObject& aNodeOrGlobalObject,
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
+  MOZ_ASSERT(aSinkGroup == kTrustedTypesOnlySinkGroup);
+
+  if (IsTrustedType(aInput)) {
+    return GetAsTrustedType(aInput);
+  }
+
+  nsIGlobalObject* globalObject = nullptr;
+  nsPIDOMWindowInner* piDOMWindowInner = nullptr;
+  RequireTrustedTypesForDirectiveState requireTrustedTypesForDirectiveState;
+  if (!PrepareExecutionOfTrustedTypesDefaultPolicy(
+          aNodeOrGlobalObject, aPrincipalOrNull, globalObject, piDOMWindowInner,
+          &requireTrustedTypesForDirectiveState, aError)) {
+    return aError.Failed() ? nullptr : GetAsString(aInput);
   }
 
   RefPtr<ExpectedType> convertedInput;
@@ -530,6 +567,8 @@ MOZ_CAN_RUN_SCRIPT inline const nsAString* GetTrustedTypesCompliantString(
   if (!convertedInput) {
     auto location = JSCallingLocation::Get();
     if (piDOMWindowInner) {
+      RefPtr<nsIContentSecurityPolicy> csp =
+          PolicyContainer::GetCSP(piDOMWindowInner->GetPolicyContainer());
       ReportSinkTypeMismatchViolations(csp, nullptr /* aCSPEventListener */,
                                        location.FileName(), location.mLine,
                                        location.mColumn, aSink, aSinkGroup,
@@ -599,20 +638,23 @@ GetTrustedTypesCompliantStringForTrustedHTML(const nsAString& aInput,
                                              const nsAString& aSink,
                                              const nsAString& aSinkGroup,
                                              const nsINode& aNode,
+                                             nsIPrincipal* aPrincipalOrNull,
                                              Maybe<nsAutoString>& aResultHolder,
                                              ErrorResult& aError) {
-  return GetTrustedTypesCompliantString<TrustedHTML>(
-      &aInput, aSink, aSinkGroup, aNode, nullptr, aResultHolder, aError);
+  return GetTrustedTypesCompliantString<TrustedHTML>(&aInput, aSink, aSinkGroup,
+                                                     aNode, aPrincipalOrNull,
+                                                     aResultHolder, aError);
 }
 
 MOZ_CAN_RUN_SCRIPT const nsAString*
 GetTrustedTypesCompliantStringForTrustedScript(
     const nsAString& aInput, const nsAString& aSink,
     const nsAString& aSinkGroup, nsIGlobalObject& aGlobalObject,
-    Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {
+    nsIPrincipal* aPrincipalOrNull, Maybe<nsAutoString>& aResultHolder,
+    ErrorResult& aError) {
   return GetTrustedTypesCompliantString<TrustedScript>(
-      &aInput, aSink, aSinkGroup, aGlobalObject, nullptr, aResultHolder,
-      aError);
+      &aInput, aSink, aSinkGroup, aGlobalObject, aPrincipalOrNull,
+      aResultHolder, aError);
 }
 
 MOZ_CAN_RUN_SCRIPT const nsAString*
@@ -620,8 +662,12 @@ GetTrustedTypesCompliantStringForTrustedScript(
     const nsAString& aInput, const nsAString& aSink,
     const nsAString& aSinkGroup, const nsINode& aNode,
     Maybe<nsAutoString>& aResultHolder, ErrorResult& aError) {
+  // Specify a null principal, as ScriptElement::UpdateTrustWorthiness() has
+  // already checked whether this is running in an isolated web extension
+  // context.
   return GetTrustedTypesCompliantString<TrustedScript>(
-      &aInput, aSink, aSinkGroup, aNode, nullptr, aResultHolder, aError);
+      &aInput, aSink, aSinkGroup, aNode, nullptr /* aPrincipalOrNull */,
+      aResultHolder, aError);
 }
 
 bool GetTrustedTypeDataForAttribute(const nsAtom* aElementName,
@@ -789,7 +835,8 @@ bool AreArgumentsTrustedForEnsureCSPDoesNotBlockStringCompilation(
     JS::Handle<JS::StackGCVector<JSString*>> aParameterStrings,
     JS::Handle<JSString*> aBodyString,
     JS::Handle<JS::StackGCVector<JS::Value>> aParameterArgs,
-    JS::Handle<JS::Value> aBodyArg, ErrorResult& aError) {
+    JS::Handle<JS::Value> aBodyArg, nsIPrincipal* aPrincipalOrNull,
+    ErrorResult& aError) {
   // EnsureCSPDoesNotBlockStringCompilation is essentially HTML's implementation
   // of HostEnsureCanCompileStrings, so we only consider the cases described in
   // the Dynamic Code Brand Checks spec. The algorithm is also supposed to be
@@ -920,8 +967,8 @@ bool AreArgumentsTrustedForEnsureCSPDoesNotBlockStringCompilation(
           codeString,
           aCompilationType == JS::CompilationType::Function ? functionSink
                                                             : evalSink,
-          kTrustedTypesOnlySinkGroup, *pinnedGlobal, compliantStringHolder,
-          aError);
+          kTrustedTypesOnlySinkGroup, *pinnedGlobal, aPrincipalOrNull,
+          compliantStringHolder, aError);
 
   // Step 2.7-2.8.
   // Callers will take care of throwing an EvalError when we return false.
