@@ -12,6 +12,7 @@ use euclid::approxeq::ApproxEq;
 use euclid::{point2, vec2, size2};
 use api::{ExtendMode, GradientStop, LineOrientation, PremultipliedColorF, ColorF, ColorU};
 use api::units::*;
+use crate::gpu_types::{ImageBrushPrimitiveData, LinearGradientBrushData};
 use crate::pattern::{Pattern, PatternBuilder, PatternBuilderContext, PatternBuilderState, PatternKind, PatternShaderInput, PatternTextureInput};
 use crate::prim_store::gradient::{gpu_gradient_stops_blocks, write_gpu_gradient_stops_tree, GradientKind};
 use crate::scene_building::IsVisible;
@@ -19,7 +20,7 @@ use crate::frame_builder::FrameBuildingState;
 use crate::intern::{Internable, InternDebug, Handle as InternHandle};
 use crate::internal_types::LayoutPrimitiveInfo;
 use crate::image_tiling::simplify_repeated_primitive;
-use crate::prim_store::{BrushSegment, GradientTileRange};
+use crate::prim_store::{BrushSegment, GradientTileRange, VECS_PER_SEGMENT};
 use crate::prim_store::{PrimitiveInstanceKind, PrimitiveOpacity};
 use crate::prim_store::{PrimKeyCommonData, PrimTemplateCommonData, PrimitiveStore};
 use crate::prim_store::{NinePatchDescriptor, PointKey, SizeKey, InternablePrimitive};
@@ -28,7 +29,6 @@ use crate::render_task_graph::RenderTaskId;
 use crate::render_task_cache::{RenderTaskCacheKeyKind, RenderTaskCacheKey, RenderTaskParent};
 use crate::renderer::{GpuBufferAddress, GpuBufferBuilder};
 use crate::segment::EdgeAaSegmentMask;
-use crate::util::pack_as_float;
 use super::{stops_and_min_alpha, GradientStopKey, GradientGpuBlockBuilder, apply_gradient_local_clip};
 use std::ops::{Deref, DerefMut};
 use std::mem::swap;
@@ -84,8 +84,8 @@ impl InternDebug for LinearGradientKey {}
 pub struct LinearGradientTemplate {
     pub common: PrimTemplateCommonData,
     pub extend_mode: ExtendMode,
-    pub start_point: DevicePoint,
-    pub end_point: DevicePoint,
+    pub start_point: LayoutPoint,
+    pub end_point: LayoutPoint,
     pub task_size: DeviceIntSize,
     pub scale: DeviceVector2D,
     pub stretch_size: LayoutSize,
@@ -111,6 +111,7 @@ impl PatternBuilder for LinearGradientTemplate {
         } else {
             (self.start_point, self.end_point)
         };
+
         linear_gradient_pattern(
             start,
             end,
@@ -396,8 +397,8 @@ impl From<LinearGradientKey> for LinearGradientTemplate {
         // should be drawn in.
         let stops_opacity = PrimitiveOpacity::from_alpha(min_alpha);
 
-        let start_point = DevicePoint::new(item.start_point.x, item.start_point.y);
-        let end_point = DevicePoint::new(item.end_point.x, item.end_point.y);
+        let start_point = LayoutPoint::new(item.start_point.x, item.start_point.y);
+        let end_point = LayoutPoint::new(item.end_point.x, item.end_point.y);
         let tile_spacing: LayoutSize = item.tile_spacing.into();
         let stretch_size: LayoutSize = item.stretch_size.into();
         let mut task_size: DeviceSize = stretch_size.cast_unit();
@@ -493,47 +494,32 @@ impl LinearGradientTemplate {
         &mut self,
         frame_state: &mut FrameBuildingState,
     ) {
-        if let Some(mut request) = frame_state.gpu_cache.request(
-            &mut self.common.gpu_cache_handle
-        ) {
+        let mut writer = frame_state.frame_gpu_data.f32.write_blocks(3 + self.brush_segments.len() * VECS_PER_SEGMENT);
 
-            // Write_prim_gpu_blocks
-            if self.cached {
-                // We are using the image brush.
-                request.push(PremultipliedColorF::WHITE);
-                request.push(PremultipliedColorF::WHITE);
-                request.push([
-                    self.stretch_size.width,
-                    self.stretch_size.height,
-                    0.0,
-                    0.0,
-                ]);
-            } else {
-                // We are using the gradient brush.
-                request.push([
-                    self.start_point.x,
-                    self.start_point.y,
-                    self.end_point.x,
-                    self.end_point.y,
-                ]);
-                request.push([
-                    pack_as_float(self.extend_mode as u32),
-                    self.stretch_size.width,
-                    self.stretch_size.height,
-                    0.0,
-                ]);
-            }
-
-            // write_segment_gpu_blocks
-            for segment in &self.brush_segments {
-                // has to match VECS_PER_SEGMENT
-                request.write_segment(
-                    segment.local_rect,
-                    segment.extra_data,
-                );
-            }
+        // Write_prim_gpu_blocks
+        if self.cached {
+            writer.push(&ImageBrushPrimitiveData {
+                color: PremultipliedColorF::WHITE,
+                background_color: PremultipliedColorF::WHITE,
+                stretch_size: self.stretch_size,
+            });
+        } else {
+            // We are using the gradient brush.
+            writer.push(&LinearGradientBrushData {
+                start: self.start_point,
+                end: self.end_point,
+                extend_mode: self.extend_mode,
+                stretch_size: self.stretch_size,
+            });
         }
 
+        // write_segment_gpu_blocks
+        for segment in &self.brush_segments {
+            segment.write_gpu_blocks(&mut writer);
+        }
+
+        self.common.gpu_buffer_address = writer.finish();
+
         // Tile spacing is always handled by decomposing into separate draw calls so the
         // primitive opacity is equivalent to stops opacity. This might change to being
         // set to non-opaque in the presence of tile spacing if/when tile spacing is handled
@@ -564,11 +550,10 @@ impl LinearGradientTemplate {
                 }),
                 false,
                 RenderTaskParent::Surface,
-                frame_state.gpu_cache,
                 &mut frame_state.frame_gpu_data.f32,
                 frame_state.rg_builder,
                 &mut frame_state.surface_builder,
-                &mut |rg_builder, _, _| {
+                &mut |rg_builder, _| {
                     rg_builder.add().init(RenderTask::new_dynamic(
                         self.task_size,
                         RenderTaskKind::FastLinearGradient(gradient),
@@ -593,11 +578,10 @@ impl LinearGradientTemplate {
                 }),
                 false,
                 RenderTaskParent::Surface,
-                frame_state.gpu_cache,
                 &mut frame_state.frame_gpu_data.f32,
                 frame_state.rg_builder,
                 &mut frame_state.surface_builder,
-                &mut |rg_builder, gpu_buffer_builder, _| {
+                &mut |rg_builder, gpu_buffer_builder| {
                     let stops = Some(GradientGpuBlockBuilder::build(
                         self.reverse_stops,
                         gpu_buffer_builder,
@@ -607,8 +591,11 @@ impl LinearGradientTemplate {
                     rg_builder.add().init(RenderTask::new_dynamic(
                         self.task_size,
                         RenderTaskKind::LinearGradient(LinearGradientTask {
-                            start: self.start_point,
-                            end: self.end_point,
+                            // Cached brush gradients are rasteried with 1 layout
+                            // pixel = 1 device pixel (regardless of potential
+                            // scaling factors).
+                            start: self.start_point.cast_unit(),
+                            end: self.end_point.cast_unit(),
                             scale: self.scale,
                             extend_mode: self.extend_mode,
                             stops: stops.unwrap(),
@@ -790,14 +777,14 @@ pub struct LinearGradientCacheKey {
 }
 
 pub fn linear_gradient_pattern(
-    start: DevicePoint,
-    end: DevicePoint,
+    start: LayoutPoint,
+    end: LayoutPoint,
     extend_mode: ExtendMode,
     stops: &[GradientStop],
     _is_software: bool,
     gpu_buffer_builder: &mut GpuBufferBuilder
 ) -> Pattern {
-    let num_blocks = 2 + gpu_gradient_stops_blocks(stops.len(), true);
+    let num_blocks = 2 + gpu_gradient_stops_blocks(stops.len());
     let mut writer = gpu_buffer_builder.f32.write_blocks(num_blocks);
     writer.push_one([
         start.x,
@@ -813,6 +800,7 @@ pub fn linear_gradient_pattern(
     ]);
 
     let is_opaque = write_gpu_gradient_stops_tree(stops, GradientKind::Linear, extend_mode, &mut writer);
+
     let gradient_address = writer.finish();
 
     Pattern {
