@@ -27,9 +27,6 @@
 #include <sys/un.h>
 #include <sys/uio.h>
 
-#include <string>
-#include <map>
-
 #include "base/command_line.h"
 #include "base/eintr_wrapper.h"
 #include "base/logging.h"
@@ -41,10 +38,8 @@
 #include "chrome/common/ipc_message_utils.h"
 #include "mozilla/ipc/Endpoint.h"
 #include "mozilla/ipc/ProtocolUtils.h"
-#include "mozilla/Atomics.h"
 #include "mozilla/StaticMutex.h"
 #include "mozilla/UniquePtr.h"
-#include "mozilla/Unused.h"
 
 // Use OS specific iovec array limit where it's possible.
 #if defined(IOV_MAX)
