@@ -20,10 +20,12 @@
 #include <string.h>
 #include <utility>
 
+#include "builtin/Array.h"
 #include "builtin/Boolean.h"
 #include "builtin/intl/CommonFunctions.h"
 #include "builtin/intl/FormatBuffer.h"
 #include "builtin/intl/LanguageTag.h"
+#include "builtin/intl/LocaleNegotiation.h"
 #include "builtin/intl/StringAsciiChars.h"
 #include "builtin/String.h"
 #include "js/Conversions.h"
@@ -139,9 +141,9 @@ static LocaleObject* CreateLocaleObject(JSContext* cx, HandleObject prototype,
     return nullptr;
   }
 
-  locale->setFixedSlot(LocaleObject::LANGUAGE_TAG_SLOT, StringValue(tagStr));
-  locale->setFixedSlot(LocaleObject::BASENAME_SLOT, StringValue(baseName));
-  locale->setFixedSlot(LocaleObject::UNICODE_EXTENSION_SLOT, unicodeExtension);
+  locale->initFixedSlot(LocaleObject::LANGUAGE_TAG_SLOT, StringValue(tagStr));
+  locale->initFixedSlot(LocaleObject::BASENAME_SLOT, StringValue(baseName));
+  locale->initFixedSlot(LocaleObject::UNICODE_EXTENSION_SLOT, unicodeExtension);
 
   return locale;
 }
@@ -1396,6 +1398,70 @@ const ClassSpec LocaleObject::classSpec_ = {
     ClassSpec::DontDefineConstructor,
 };
 
+static JSLinearString* ValidateAndCanonicalizeLanguageTag(
+    JSContext* cx, Handle<JSLinearString*> string) {
+  // Handle the common case (a standalone language) first.
+  // Only the following Unicode BCP 47 locale identifier subset is accepted:
+  //   unicode_locale_id = unicode_language_id
+  //   unicode_language_id = unicode_language_subtag
+  //   unicode_language_subtag = alpha{2,3}
+  JSLinearString* language;
+  JS_TRY_VAR_OR_RETURN_NULL(cx, language,
+                            intl::ParseStandaloneISO639LanguageTag(cx, string));
+  if (language) {
+    return language;
+  }
+
+  mozilla::intl::Locale tag;
+  if (!intl::ParseLocale(cx, string, tag)) {
+    return nullptr;
+  }
+
+  auto result = tag.Canonicalize();
+  if (result.isErr()) {
+    if (result.unwrapErr() ==
+        mozilla::intl::Locale::CanonicalizationError::DuplicateVariant) {
+      JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
+                                JSMSG_DUPLICATE_VARIANT_SUBTAG);
+    } else {
+      intl::ReportInternalError(cx);
+    }
+    return nullptr;
+  }
+
+  intl::FormatBuffer<char, intl::INITIAL_CHAR_BUFFER_SIZE> buffer(cx);
+  if (auto result = tag.ToString(buffer); result.isErr()) {
+    intl::ReportInternalError(cx, result.unwrapErr());
+    return nullptr;
+  }
+
+  return buffer.toAsciiString(cx);
+}
+
+static JSLinearString* ValidateAndCanonicalizeLanguageTag(
+    JSContext* cx, Handle<Value> tagValue) {
+  if (tagValue.isObject()) {
+    JSString* tagStr;
+    JS_TRY_VAR_OR_RETURN_NULL(
+        cx, tagStr,
+        LanguageTagFromMaybeWrappedLocale(cx, &tagValue.toObject()));
+    if (tagStr) {
+      return tagStr->ensureLinear(cx);
+    }
+  }
+
+  JSString* tagStr = ToString(cx, tagValue);
+  if (!tagStr) {
+    return nullptr;
+  }
+
+  Rooted<JSLinearString*> tagLinearStr(cx, tagStr->ensureLinear(cx));
+  if (!tagLinearStr) {
+    return nullptr;
+  }
+  return ValidateAndCanonicalizeLanguageTag(cx, tagLinearStr);
+}
+
 bool js::intl_ValidateAndCanonicalizeLanguageTag(JSContext* cx, unsigned argc,
                                                  Value* vp) {
   CallArgs args = CallArgsFromVp(argc, vp);
@@ -1420,57 +1486,10 @@ bool js::intl_ValidateAndCanonicalizeLanguageTag(JSContext* cx, unsigned argc,
     return true;
   }
 
-  JSString* tagStr = ToString(cx, tagValue);
-  if (!tagStr) {
-    return false;
-  }
-
-  Rooted<JSLinearString*> tagLinearStr(cx, tagStr->ensureLinear(cx));
-  if (!tagLinearStr) {
-    return false;
-  }
-
-  // Handle the common case (a standalone language) first.
-  // Only the following Unicode BCP 47 locale identifier subset is accepted:
-  //   unicode_locale_id = unicode_language_id
-  //   unicode_language_id = unicode_language_subtag
-  //   unicode_language_subtag = alpha{2,3}
-  JSString* language;
-  JS_TRY_VAR_OR_RETURN_FALSE(
-      cx, language, intl::ParseStandaloneISO639LanguageTag(cx, tagLinearStr));
-  if (language) {
-    args.rval().setString(language);
-    return true;
-  }
-
-  mozilla::intl::Locale tag;
-  if (!intl::ParseLocale(cx, tagLinearStr, tag)) {
-    return false;
-  }
-
-  auto result = tag.Canonicalize();
-  if (result.isErr()) {
-    if (result.unwrapErr() ==
-        mozilla::intl::Locale::CanonicalizationError::DuplicateVariant) {
-      JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
-                                JSMSG_DUPLICATE_VARIANT_SUBTAG);
-    } else {
-      intl::ReportInternalError(cx);
-    }
-    return false;
-  }
-
-  intl::FormatBuffer<char, intl::INITIAL_CHAR_BUFFER_SIZE> buffer(cx);
-  if (auto result = tag.ToString(buffer); result.isErr()) {
-    intl::ReportInternalError(cx, result.unwrapErr());
-    return false;
-  }
-
-  JSString* resultStr = buffer.toAsciiString(cx);
+  auto* resultStr = ValidateAndCanonicalizeLanguageTag(cx, tagValue);
   if (!resultStr) {
     return false;
   }
-
   args.rval().setString(resultStr);
   return true;
 }
@@ -1617,3 +1636,97 @@ bool js::intl_ValidateAndCanonicalizeUnicodeExtensionType(JSContext* cx,
   args.rval().setString(result);
   return true;
 }
+
+/**
+ * Canonicalizes a locale list.
+ *
+ * Spec: ECMAScript Internationalization API Specification, 9.2.1.
+ */
+bool js::intl::CanonicalizeLocaleList(JSContext* cx, Handle<Value> locales,
+                                      MutableHandle<LocalesList> result) {
+  MOZ_ASSERT(result.empty());
+
+  // Step 1.
+  if (locales.isUndefined()) {
+    return true;
+  }
+
+  // Step 3 (and the remaining steps).
+  if (locales.isString()) {
+    Rooted<JSLinearString*> linear(cx, locales.toString()->ensureLinear(cx));
+    if (!linear) {
+      return false;
+    }
+
+    auto* languageTag = ValidateAndCanonicalizeLanguageTag(cx, linear);
+    if (!languageTag) {
+      return false;
+    }
+    return result.append(languageTag);
+  }
+
+  if (locales.isObject()) {
+    JSString* languageTag;
+    JS_TRY_VAR_OR_RETURN_FALSE(
+        cx, languageTag,
+        LanguageTagFromMaybeWrappedLocale(cx, &locales.toObject()));
+    if (languageTag) {
+      auto* linear = languageTag->ensureLinear(cx);
+      if (!linear) {
+        return false;
+      }
+      return result.append(linear);
+    }
+  }
+
+  // Step 2. (Implicit)
+
+  // Step 4.
+  Rooted<JSObject*> obj(cx, ToObject(cx, locales));
+  if (!obj) {
+    return false;
+  }
+
+  // Step 5.
+  uint64_t length;
+  if (!GetLengthProperty(cx, obj, &length)) {
+    return false;
+  }
+
+  // Steps 6-7.
+  Rooted<Value> value(cx);
+  for (uint64_t k = 0; k < length; k++) {
+    // Step 7.a-c.
+    bool hole;
+    if (!CheckForInterrupt(cx) ||
+        !HasAndGetElement(cx, obj, k, &hole, &value)) {
+      return false;
+    }
+
+    if (!hole) {
+      // Step 7.c.ii.
+      if (!value.isString() && !value.isObject()) {
+        JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
+                                  JSMSG_INVALID_LOCALES_ELEMENT);
+        return false;
+      }
+
+      // Step 7.c.iii-iv.
+      JSLinearString* tag = ValidateAndCanonicalizeLanguageTag(cx, value);
+      if (!tag) {
+        return false;
+      }
+
+      // Step 7.c.v.
+      bool addToResult =
+          std::none_of(result.begin(), result.end(),
+                       [tag](auto* other) { return EqualStrings(tag, other); });
+      if (addToResult && !result.append(tag)) {
+        return false;
+      }
+    }
+  }
+
+  // Step 8.
+  return true;
+}
