@@ -16,6 +16,7 @@
 
 #include "builtin/Array.h"
 #include "builtin/intl/CommonFunctions.h"
+#include "builtin/intl/LocaleNegotiation.h"
 #include "builtin/intl/StringAsciiChars.h"
 #include "gc/AllocKind.h"
 #include "gc/GCContext.h"
@@ -41,6 +42,7 @@
 #include "vm/NativeObject-inl.h"
 
 using namespace js;
+using namespace js::intl;
 
 const JSClassOps SegmenterObject::classOps_ = {
     nullptr,                    // addProperty
@@ -66,6 +68,9 @@ const JSClass SegmenterObject::class_ = {
 
 const JSClass& SegmenterObject::protoClass_ = PlainObject::class_;
 
+static bool segmenter_supportedLocalesOf(JSContext* cx, unsigned argc,
+                                         Value* vp);
+
 static bool segmenter_toSource(JSContext* cx, unsigned argc, Value* vp) {
   CallArgs args = CallArgsFromVp(argc, vp);
   args.rval().setString(cx->names().Segmenter);
@@ -73,8 +78,7 @@ static bool segmenter_toSource(JSContext* cx, unsigned argc, Value* vp) {
 }
 
 static const JSFunctionSpec segmenter_static_methods[] = {
-    JS_SELF_HOSTED_FN("supportedLocalesOf", "Intl_Segmenter_supportedLocalesOf",
-                      1, 0),
+    JS_FN("supportedLocalesOf", segmenter_supportedLocalesOf, 1, 0),
     JS_FS_END,
 };
 
@@ -1023,3 +1027,20 @@ bool js::intl_FindNextSegmentBoundaries(JSContext* cx, unsigned argc,
   args.rval().setObject(*result);
   return true;
 }
+
+/**
+ * Intl.Segmenter.supportedLocalesOf ( locales [ , options ] )
+ */
+static bool segmenter_supportedLocalesOf(JSContext* cx, unsigned argc,
+                                         Value* vp) {
+  CallArgs args = CallArgsFromVp(argc, vp);
+
+  // Steps 1-3.
+  auto* array = SupportedLocalesOf(cx, AvailableLocaleKind::Segmenter,
+                                   args.get(0), args.get(1));
+  if (!array) {
+    return false;
+  }
+  args.rval().setObject(*array);
+  return true;
+}
